<?php
/**
 *
 * @copyright 2013 Spirit of Co.,Ltd. All Rights Reserved.
 * @link http://sbps.ec-cube.pro/
 *
 */
require_once(MDL_SPS_HELPEREX_PATH . 'SC_Mdl_SPS_Helper_Base_Ex.php');

/**
 * 決済個別用ヘルパー関数 CreditRegistApiクラス
 */
class SC_Mdl_SPS_Helper_CreditRegistApi extends SC_Mdl_SPS_Helper_Base_Ex {

    function init() {
        $this->payment_type_id = SPS_CREDIT_API_PAYID;
        $this->payment_name = 'CreditRegistApi';
    }

    function getPayMethod() {
        return 'creditregistapi';
    }

    function getPaymentBlocName() {
        return 'sps_creditregistapi';
    }

    function getConnectType() {
        return MDL_SPS_CONNECT_TYPE_API;
    }

    function getSpsPayType() {
        // 購入タイプ(固定　0:都度購入、1:継続購入)
        return '0';
    }

    function getPaymentRequestApiId() {
        return 'ST01-00111-101';
    }

    function getPaymentCommitApiId() {
        return 'ST02-00101-101';
    }

    function isSales() {
        return true;
    }

    function isCancel() {
        return true;
    }

    function isReAuth() {
        return true;
    }

    function isGetStatus() {
        return false;
//        return true;
    }

    function isThreePhase() {
        return true;
    }

    function getPaymentRuleDefault() {
        $arrData = parent::getPaymentRuleDefault();
        $arrData['rule_max'] = '2';
//        $arrData['upper_rule'] = '9999999';
//        $arrData['upper_rule_max'] = $arrData['upper_rule'];
        return $arrData;
    }

    function initParamPaymentInput(&$objPage, &$objFormParam, &$arrConfig) {
        parent::initParamPaymentInput($objPage, $objFormParam, $arrConfig);

        $objPage->arrAuthType = array(
                SPS_AUTH_DATE_COMMIT => '売上日指定要求(与信のみ)',
                SPS_AUTH_AUTO => '自動売上',
            );
        $objPage->arrOptionActive = array('1' => '要', '0' => '不要');

        $objPage->arrDealingsTypes = array(
            '10' =>'一括',
//            '21' =>'ボーナス一括',
            '61' =>'分割',
            '80' =>'リボ',
            );
        if (MDL_SPS_CREDIT_BONUS_ENABLE) {
            $objPage->arrDealingsTypes[ '21' ] = 'ボーナス一括';
        }
        $objPage->arrDivideTimes = array();
//        for($i = 3; $i < 99; $i++) {
//            $objPage->arrDivideTimes[ $i ] = $i ."回";
//        }
        $arrDivideTimes = array(3,5,6,10,12,15,18,20,24);
        foreach ($arrDivideTimes as $i) {
            $objPage->arrDivideTimes[ $i ] = $i ."回";
        }

        $key = 'sps_auth_type';
        $objFormParam->addParam('売上計上設定', $key, 1, 'n',
                array('EXIST_CHECK','MAX_LENGTH_CHECK', 'NUM_CHECK'),
                SC_Utils_Ex::isBlank($arrConfig[$key]) ? '0' : $arrConfig[$key]);

        $key = 'auth_order_status';
        $objFormParam->addParam('与信後の受注状況', $key, INT_LEN, 'n',
                array('EXIST_CHECK','MAX_LENGTH_CHECK', 'NUM_CHECK'),
                SC_Utils_Ex::isBlank($arrConfig[$key]) ? ORDER_NEW : $arrConfig[$key]);

        $key = 'commit_order_status';
        $objFormParam->addParam('売上確定後の受注状況', $key, INT_LEN, 'n',
                array('MAX_LENGTH_CHECK', 'NUM_CHECK'),
                SC_Utils_Ex::isBlank($arrConfig[$key]) ? ORDER_PRE_END : $arrConfig[$key]);

        $key = 'cancel_order_status';
        $objFormParam->addParam('与信取消後の受注状況', $key, INT_LEN, 'n',
                array('MAX_LENGTH_CHECK', 'NUM_CHECK'),
                SC_Utils_Ex::isBlank($arrConfig[$key]) ? ORDER_CANCEL : $arrConfig[$key]);

    }

    function checkErrorPaymentInput(&$objFormParam) {
        return parent::checkErrorPaymentInput($objFormParam);
    }

    function initParam(&$objFormParam, &$arrPayment, &$arrOrder) {
        parent::initParam($objFormParam, $arrPayment, $arrOrder);
    }

    /**
     * 入力内容のチェックを行なう.
     *
     * @param SC_FormParam $objFormParam SC_FormParam インスタンス
     * @return array 入力チェック結果の配列
     */
    function checkError(&$objFormParam) {
        $arrParam = $objFormParam->getHashArray();
        $objErr = new SC_CheckError_Ex($arrParam);
        $objErr->arrErr = $objFormParam->checkError();

        return $objErr->arrErr;
    }

    function getApiPayMethodInfoData() {
        return '';
    }


    function modeAction($mode, &$objFormParam, &$arrOrder, $objPage) {
        switch($mode) {
        case 'return':
            break;
        case 'next':
            break;
        case 'spsreturn':
            break;
        default:
            $objPage->tpl_payment_onload = 'fnAutoLoadSubmit();';
            break;
        }
        parent::modeAction($mode, $objFormParam, $arrOrder, $objPage);
    }

}
